@@ -1,6 +1,8 @@
 package cmd
 
 import (
+	"context"
+	"errors"
 	"fmt"
 	"github.com/mzz2017/gg/tracer"
 	"github.com/sirupsen/logrus"
@@ -8,6 +10,9 @@ import (
 	"github.com/spf13/viper"
 	"os"
 	"os/exec"
+	"os/signal"
+	"runtime"
+	"syscall"
 )
 
 var (
@@ -21,7 +26,8 @@ var (
 program to your modern proxy without installing any other programs.`,
 		Version: Version,
 		Run: func(cmd *cobra.Command, args []string) {
-			if len(args) == 0 {
+			hasSelectFlag, _ := cmd.PersistentFlags().GetBool("select")
+			if len(args) == 0 && !hasSelectFlag {
 				fmt.Println(`No command is given, you can try:
 $ gg --help
 or
@@ -30,11 +36,19 @@ $ gg git clone https://github.com/mzz2017/gg.git`)
 			}
 			// initiate config from args and config file
 			log := GetLogger(verbose)
+			log.Traceln("Version:", Version)
+			log.Tracef("OS/Arch: %v/%v\n", runtime.GOOS, runtime.GOARCH)
 			v, _ = getConfig(log, true, viper.New, cmd)
 			// validate command and get the fullPath from $PATH
-			fullPath, err := exec.LookPath(args[0])
-			if err != nil {
-				logrus.Fatal("exec.LookPath:", err)
+			var (
+				fullPath string
+				err      error
+			)
+			if len(args) != 0 {
+				fullPath, err = exec.LookPath(args[0])
+				if err != nil {
+					logrus.Fatal("exec.LookPath:", err)
+				}
 			}
 			// get dialer
 			dialer, err := GetDialer(log)
@@ -42,13 +56,16 @@ $ gg git clone https://github.com/mzz2017/gg.git`)
 				logrus.Fatal("GetDialer:", err)
 			}
 
+			if len(args) == 0 {
+				return
+			}
+
 			noUDP, err := cmd.Flags().GetBool("noudp")
 			if err != nil {
 				logrus.Fatal("GetBool(noudp):", err)
 			}
 			if !noUDP && !dialer.SupportUDP() {
-				log.Warn("Your proxy server does not support UDP, so we will not redirect UDP traffic.")
-				noUDP = true
+				log.Info("Your proxy server does not support UDP, so we will not redirect UDP traffic.")
 			}
 			preserveEnv, err := cmd.Flags().GetBool("preserve-env")
 			if err != nil {
@@ -58,7 +75,10 @@ $ gg git clone https://github.com/mzz2017/gg.git`)
 			if preserveEnv {
 				env = os.Environ()
 			}
+			ctx, cancel := context.WithCancel(context.Background())
+			defer cancel()
 			t, err := tracer.New(
+				ctx,
 				fullPath,
 				args,
 				&os.ProcAttr{Files: []*os.File{os.Stdin, os.Stdout, os.Stderr}, Env: env},
@@ -69,9 +89,18 @@ $ gg git clone https://github.com/mzz2017/gg.git`)
 			if err != nil {
 				logrus.Fatal("tracer.New:", err)
 			}
+			go func() {
+				// listen signal
+				sigs := make(chan os.Signal, 1)
+				signal.Notify(sigs, syscall.SIGINT, syscall.SIGTERM, syscall.SIGHUP, syscall.SIGQUIT, syscall.SIGKILL, syscall.SIGILL)
+				<-sigs
+				cancel()
+			}()
 			code, err := t.Wait()
 			if err != nil {
-				logrus.Fatal("tracer.Wait:", err)
+				if !errors.Is(err, context.Canceled) {
+					logrus.Fatal("tracer.Wait:", err)
+				}
 			}
 			os.Exit(code)
 		},
@@ -97,7 +126,7 @@ func init() {
 
 func GetLogger(verbose int) *logrus.Logger {
 	log := logrus.New()
-	log.SetLevel(logrus.InfoLevel)
+	log.SetLevel(logrus.WarnLevel)
 	if verbose > 0 {
 		if verbose == 1 {
 			log.SetLevel(logrus.InfoLevel)
