@@ -4,6 +4,7 @@ import (
 	"encoding/base64"
 	"fmt"
 	"github.com/fatih/structs"
+	"net/url"
 	"reflect"
 	"strings"
 )
@@ -28,6 +29,13 @@ func BoolToString(b bool) string {
 	return "false"
 }
 
+func StringToBool(s string) bool {
+	if strings.EqualFold(s, "true") || s == "1" {
+		return true
+	}
+	return false
+}
+
 func Base64URLDecode(s string) (string, error) {
 	s = strings.TrimSpace(s)
 	saver := s
@@ -91,3 +99,17 @@ func SliceUint64toUint32(from []uint64) (to []uint32) {
 	}
 	return to
 }
+
+func SetValue(values *url.Values, key string, value string) {
+	if value == "" {
+		return
+	}
+	values.Set(key, value)
+}
+
+func Max(a, b int) int {
+	if a > b {
+		return a
+	}
+	return b
+}
