@@ -1,16 +1,15 @@
 package tracer
 
 import (
+	"context"
 	"fmt"
+	"github.com/mzz2017/gg/dialer"
 	"github.com/mzz2017/gg/proxy"
 	"github.com/sirupsen/logrus"
-	proxy2 "golang.org/x/net/proxy"
-	"golang.org/x/sys/unix"
 	"os"
 	"runtime"
 	"syscall"
 	"time"
-	"unsafe"
 )
 
 type SocketMetadata struct {
@@ -20,10 +19,12 @@ type SocketMetadata struct {
 
 // Tracer is not thread-safe.
 type Tracer struct {
+	ctx        context.Context
 	ignoreUDP  bool
+	supportUDP bool
 	log        *logrus.Logger
 	proxy      *proxy.Proxy
-	mainPID    int
+	proc       *os.Process
 	storehouse Storehouse
 	socketInfo map[int]map[int]SocketMetadata
 	closed     chan struct{}
@@ -31,9 +32,11 @@ type Tracer struct {
 	exitErr    error
 }
 
-func New(name string, argv []string, attr *os.ProcAttr, dialer proxy2.Dialer, ignoreUDP bool, logger *logrus.Logger) (*Tracer, error) {
+func New(ctx context.Context, name string, argv []string, attr *os.ProcAttr, dialer *dialer.Dialer, ignoreUDP bool, logger *logrus.Logger) (*Tracer, error) {
 	t := &Tracer{
+		ctx:        ctx,
 		log:        logger,
+		supportUDP: dialer.SupportUDP(),
 		proxy:      proxy.New(logger, dialer),
 		socketInfo: make(map[int]map[int]SocketMetadata),
 		storehouse: MakeStorehouse(),
@@ -64,9 +67,10 @@ func New(name string, argv []string, attr *os.ProcAttr, dialer proxy2.Dialer, ig
 			t.exitErr = err
 			return
 		}
+		t.proc = proc
 		close(done)
 		time.Sleep(1 * time.Millisecond)
-		code, err := t.trace(proc.Pid)
+		code, err := t.trace()
 		t.exitCode = code
 		t.exitErr = err
 		close(t.closed)
@@ -84,7 +88,8 @@ func (t *Tracer) Wait() (exitCode int, err error) {
 }
 
 // Trace traces the process. proc is the process ID (main thread).
-func (t *Tracer) trace(proc int) (exitCode int, err error) {
+func (t *Tracer) trace() (exitCode int, err error) {
+	proc := t.proc.Pid
 	// Thanks https://stackoverflow.com/questions/5477976/how-to-ptrace-a-multi-threaded-application and https://github.com/hmgle/graftcp
 	err = syscall.PtraceAttach(proc)
 	if err != nil {
@@ -110,12 +115,26 @@ func (t *Tracer) trace(proc int) (exitCode int, err error) {
 	}
 	//t.log.Tracef("child %v created\n", proc)
 	for {
+		select {
+		case <-t.ctx.Done():
+			syscall.PtraceDetach(proc)
+			t.proc.Kill()
+			return 1, t.ctx.Err()
+		default:
+		}
 		var status syscall.WaitStatus
 		child, err := syscall.Wait4(-1, &status, syscall.WALL, nil)
 		if err != nil {
 			return 0, fmt.Errorf("wait4() threw: %w", err)
 		}
-		t.log.Tracef("main: %v, child: %v\n", proc, child)
+		select {
+		case <-t.ctx.Done():
+			syscall.PtraceDetach(proc)
+			t.proc.Kill()
+			return 1, t.ctx.Err()
+		default:
+		}
+		//t.log.Tracef("main: %v, child: %v\n", proc, child)
 		if t.getSocketInfo(child, 0) == nil {
 			t.saveSocketInfo(child, 0, SocketMetadata{
 				Family: syscall.AF_LOCAL,
@@ -150,7 +169,7 @@ func (t *Tracer) trace(proc int) (exitCode int, err error) {
 				var regs syscall.PtraceRegs
 				err = ptraceGetRegs(child, &regs)
 				if err == nil {
-					t.log.Tracef("pid: %v, inst: %v", child, inst(&regs))
+					//t.log.Tracef("pid: %v, inst: %v", child, inst(&regs))
 					entryStop := isEntryStop(&regs)
 					if entryStop {
 						if err := t.entryHandler(child, &regs); err != nil {
@@ -165,14 +184,15 @@ func (t *Tracer) trace(proc int) (exitCode int, err error) {
 					t.log.Tracef("PtraceGetRegs: %v", err)
 				}
 			default:
-				var sigInfo unix.SignalfdSiginfo
-				if e := ptrace(syscall.PTRACE_GETSIGINFO, child, 0, uintptr(unsafe.Pointer(&sigInfo))); e == nil {
-					sig = int(sigInfo.Signo)
-				}
 				// urgent I/O condition, window changed, etc.
-				t.log.Tracef("%v: stopped: %v", child, signal)
-				if signal == syscall.SIGILL {
-					t.log.Errorf("%v: stopped: %v", child, signal)
+				sig = int(signal)
+				switch signal {
+				case syscall.SIGSTOP:
+					sig = 0
+				case syscall.SIGILL:
+					t.log.Errorf("%v: %v", child, signal)
+				default:
+					t.log.Tracef("%v: %v", child, signal)
 				}
 			}
 		}
